@@ -6,14 +6,18 @@ use constant PUBLIC => "Public";
 use constant MEMBER => "Member";
 use constant OWNER  => "Owner";
 use constant LOGOUT => "/logout";
+use constant LOGIN  => "/login";
+use constant ANY    => "Any";
+use constant ANONYMOUS => "UNKNOWN";
 
 use FindBin;
 use Exporter;
+use Carp;
 use vars qw( @ISA @EXPORT_OK *FILE );
 use strict;
 
 @ISA	    = "Exporter";
-@EXPORT_OK  = qw( PERMIT DENY PUBLIC MEMBER OWNER LOGOUT );
+@EXPORT_OK  = qw( PERMIT DENY PUBLIC MEMBER OWNER LOGIN LOGOUT ANY ANONYMOUS );
 
 my %Defaults = (
     ### Gateway server networking values.
@@ -21,14 +25,20 @@ my %Defaults = (
     GatewayPort	    => 5280, 
     PollInterval    => 10,
     ListenQueue	    => 10,
+    HandleTimeout   => 3,
+    IdleTimeout     => 300,
+    MaxMissedARP    => 2,
+    ForkOff	    => 1,
 
     ### No. of seconds before logins/renewals expire.
     LoginTimeout    => 300,
+    MinLoginTimeout => 60,
 
     ### Fraction of LoginTimeout to loiter before renewing.
     RenewTimeout    => .75,
 
     ### Authservice networking values.
+    DataSource	    => "DBI",
     NotifyTimeout   => 30,
 
     ### GPG Locations. Assumes it's in your path.
@@ -78,8 +88,12 @@ sub file {
     $filename = $self->{$filename}
 	if $self->{$filename};
 
+    # Append the DocumentRoot if set and if the 
+    # filename isn't absolute or it doesn't exist.
+    #
     $filename = "$self->{DocumentRoot}/$filename"
-	if $self->{DocumentRoot} and not -r $filename;    
+	if $self->{DocumentRoot}
+	and ( $filename !~ /^\//o or not -r $filename );
 
     open( FILE, "<$filename" )
 	or return $self->log( 1, "file $filename: $!" );
@@ -105,7 +119,8 @@ sub parse {
 	    next unless $line =~ /^\w/o;
 
 	    # Split key / value pairs.
-	    push @pairs, split /\s+/, $line, 2;
+	    my ($key, $val) = split( /\s+/, $line, 2 );
+	    push @pairs, $key, $val;
 	}
     }
 
@@ -123,11 +138,11 @@ sub deparse {
 sub read_config {
     my ( $self, $filename ) = @_;
 
-    die "No config file specified! Does \$NOCAT point to your nocat.conf?\n"
+    croak "No config file specified! Does \$NOCAT point to your nocat.conf?\n"
 	unless $filename;
 
     my $file	= $self->file( $filename ) 
-	or die "Can't read config file $filename: $!";
+	or croak "Can't read config file $filename: $!";
 
     my %args	= $self->parse( $file );
 
@@ -190,6 +205,15 @@ sub url_decode {
     return wantarray ? @args : $args[0];
 }
 
+sub url {
+    my ( $self, $url, $args )	= @_;
+    my %data = $self->url_encode( %$args );
+    $url  = $self->format( $self->{$url} || $url );
+    $url .= ( $url =~ /\?/o ? "&" : "?" );
+    $url .= join("&", map( "$_=$data{$_}", keys %data ));
+    return $url;
+}
+
 sub format {
     my ( $self, $string, $extra ) = @_;
 
@@ -208,34 +232,74 @@ sub template {
     return $self->format( $file, $extra ); 
 }
 
-sub gateway {
-    my $self	  = shift;
+sub md5_hash {
+    my ( $self, $string, $salt ) = @_;
 
-    # We have to make a bootstrap object in order to load
-    # the config file and figure
-    # out which subclass to load. This is arguably the wrong
-    # way of doing it and should be fixed.
-
-    unless ( ref $self ) {
-        require NoCat::Gateway;
-        $self = NoCat::Gateway->new( @_ );
+    unless ( $salt ) {
+	my @chars = ( "0".."9", "a".."z", "A".."Z", ".", "/" );
+	$salt  = "";
+	$salt .= @chars[int rand @chars] for ( 1 .. 8 );
     }
 
-    my $class = "NoCat::Gateway::$self->{GatewayMode}";
+    $salt = '$1$' . substr( $salt, 0, 8 ) if $salt !~ /^\$1\$/o;
 
-    $self->log( 1, "GatewayMode parameter contains invalid characters" )
-    	if $class =~ s/[^\w:]//gos;
+    return crypt( $string, $salt );
+}
 
-    eval "require $class"
-	or die "Can't run in '$self->{GatewayMode}' mode: $@";
+sub increment_token {
+    my ( $self, $token ) = @_;
+    my $salt = ++substr( $token, -8 );
+    return $self->md5_hash( $token, $salt );
+}
+
+sub instantiate {
+    my $self	= shift;
+    my $class 	= shift;
+    my ( $super, $config );
+
+    if ( $super = ref $self ) {
+	# $self is an object, which presumably already has the config data.
+	$config = $self;	
+    } else {
+	# Gotta instantiate a bootstrap object to load up the config data.
+	$config = __PACKAGE__->new( @_ );
+	$super  = $self;
+    }
+
+    $class = "$super\::$config->{$class}";
+
+    croak "Source class $class contains invalid characters"
+	if $class =~ y/A-Za-z0-9_://cd;
+
+    eval "require $class" or
+        croak "Can't load class '$class': $@";
 
     return $class->new( Parent => $self, @_ );
 }
 
+sub gateway {
+    my $self	  = shift;
+    require NoCat::Gateway;
+    return NoCat::Gateway->new( Parent => $self, @_ );
+}
+
+my $Firewall; # Singleton instance.
+
 sub firewall {
     my $self = shift;
-    require NoCat::Firewall;
-    return NoCat::Firewall->new( Parent => $self, @_ );
+
+    # If we already have an initialized NoCat::Firewall object,
+    # use it, rather than creating a new one and having to go
+    # through the autodetect process all over again.
+
+    if ( $Firewall ) {
+	my %args = @_;
+	%$Firewall = (%$Firewall, @_);
+    } else {
+	require NoCat::Firewall;
+	$Firewall = NoCat::Firewall->new( Parent => $self, @_ );
+    }
+    return $Firewall;
 }
 
 sub auth_service {
@@ -244,12 +308,24 @@ sub auth_service {
     return NoCat::AuthService->new( Parent => $self, @_ );
 }
 
+sub source {
+    my $self = shift;
+    require NoCat::Source;
+    return NoCat::Source->new( Parent => $self, @_ );
+}
+
 sub user {
     my $self = shift;
     require NoCat::User;
     return NoCat::User->new( Parent => $self, @_ );
 }
 
+sub group {
+    my $self = shift;
+    require NoCat::Group;
+    return NoCat::Group->new( Parent => $self, @_ );
+}
+
 sub message {
     my $self = shift;
     unshift @_, "Msg" if @_ == 1;
