@@ -1,8 +1,8 @@
 package NoCat::Peer;
 
-use NoCat qw( PUBLIC );
-use strict;
+use NoCat qw( PUBLIC ANY );
 use vars qw( @ISA @REQUIRED );
+use strict;
 
 @REQUIRED   = qw( LoginTimeout );
 @ISA	    = 'NoCat';
@@ -22,11 +22,18 @@ sub socket {
     my ( $self, $sock ) = @_;
     if ( defined $sock ) {
 	$self->{Socket} = $sock;
+	$self->gateway_ip( $sock->sockhost );
 	$self->ip; # seed IP address.
     }
     return $self->{Socket};
 }
 
+sub gateway_ip {
+    my ( $self, $addr ) = @_;
+    $self->{GatewayAddr} = $addr if defined $addr;
+    return $self->{GatewayAddr};
+}
+
 sub ip {
     my ( $self, $ip ) = @_;
     $self->{IP} = $ip if defined $ip;
@@ -52,8 +59,20 @@ sub mac {
     return $self->{MAC};
 }
 
+sub id {
+    my $self = shift;
+    return $self->mac || $self->ip;
+}
+
+sub connect_time {
+    my $self = shift;
+    $self->{ConnectTime} ||= time;
+    return $self->{ConnectTime};
+}
+
 sub timestamp {
     my ( $self, $reset ) = @_;
+    $self->connect_time; # Seed ConnectTime...
     $self->{Timestamp} = time + $self->{LoginTimeout} 
 	if defined $reset or not defined $self->{Timestamp};
     return $self->{Timestamp};
@@ -84,8 +103,14 @@ sub heartbeat {
 
 sub token {
     my ( $self, $reset ) = @_;
-    $self->{Token} = sprintf( "%x", int rand 0xFFFFFFFF )
-        if defined $reset or not defined $self->{Token};
+    my $token = $self->{Token};
+    my $salt;
+
+    if ( defined $reset or not defined $token ) {
+	$token = int rand 0xFFFFFFFF unless $token;
+	$self->{Token} = $self->increment_token( $token );
+    }
+
     return $self->{Token};
 }
 
@@ -111,8 +136,15 @@ sub class {
 
 sub groups {
     my ( $self, $groups ) = @_;
-    $self->{Groups} = [ grep($_, split( /\W+/, $groups )) ] if defined $groups;
-    return @{$self->{Groups}};
+    
+    # Every user who is a member of *some* group is automatically a 
+    # member of the magical "Any" group.
+    #
+    my $list = $self->{Groups} ||= [];
+    @$list = grep $_, split( /\W+/, $groups ), ANY
+	if $groups;
+
+    return @$list;
 }
 
 1;
